// Funes. Copyright (c) 2001, Nelson Minar <nelson@monkey.org>
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package org.monkey.nelson.funes;
import org.apache.log4j.Category;

/**
 ** @author Nelson Minar &lt;nelson@monkey.org&gt;
 **/

import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Enumeration;
import java.net.URL;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.StopAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.search.Searcher;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.Hits;
import org.apache.lucene.queryParser.QueryParser;

public class Search extends Object {
    public static Category L = Category.getInstance(Search.class.getName());

    public static void main(String[] args) throws Exception {
        // Do any global initialization that's necessary
        org.monkey.nelson.protocol.xfilerange.Handler.install();

        String [] queryArgs = args;
        String queryString = null;
        try {
            queryString = makeQueryString(queryArgs);
        } catch (IllegalArgumentException iae) {
            L.error("Error making query string.", iae);
            System.err.println("Error with query string.");
            System.err.println(usageString());
            System.exit(1);
        }

        Searcher searcher = new IndexSearcher("build/index");
        Analyzer analyzer = Schema.makeAnalyzer();

        // Perform the actual query
        long timePreQuery = System.currentTimeMillis();
        Query query = QueryParser.parse(queryString, "body", analyzer);

        Hits hits = searcher.search(query);
        long timeForQuery = System.currentTimeMillis() - timePreQuery;
        L.debug("Time for query: " + timeForQuery + "ms.");

        for (int i = 0; i < hits.length(); i++) {
            Document doc = hits.doc(i);
            String author = doc.get(Schema.authorName);
            String recipients = doc.get(Schema.recipientsName);
            String date = doc.get(Schema.dateName);
            String subject = doc.get(Schema.subjectName);

            System.out.println("From: " + author +
                               "\nTo: " + recipients +
                               "\nDate: " + Schema.parseDateField(date) +
                               "\nSubject: " + subject +
                               "\nScore: " + hits.score(i)
                               + "\n");
            printMessage(hits.doc(i).get(Schema.locationName));
            System.out.println();
        }
        searcher.close();

        System.exit(0);
    }

    /** Given a URL for a message, retrieve it and print it out.
     **/
    private static void printMessage(String location) throws Exception {
        URL matchURL = new URL(location);
        InputStream is = matchURL.openStream();
        BufferedReader br = new BufferedReader(new InputStreamReader(is));

        // Read all the lines off the stream. Print out only the body.
        String s;
        boolean inHeaders = true;
        while ((s = br.readLine()) != null) {
            if (inHeaders) {
                if (s.equals(""))
                    inHeaders = false;
            } else {
                System.out.println(s);
            }
        }
    }

    private static String usageString() {
        return "Usage: Search query string\n";
    }

    /** Given an array of strings, turn it into a single query string.
     ** Currently, simply concatenates all the strings together separated
     ** by a single space.
     **/
    public static String makeQueryString(String [] queryArgs) throws IllegalArgumentException {
        if (queryArgs.length == 0)
            throw new IllegalArgumentException("query args must have at least one element.");

        StringBuffer queryStringBuffer = new StringBuffer(500);
        for (int i = 0; i < queryArgs.length; i++) {
            if (i > 0)
                queryStringBuffer.append(' ');
            queryStringBuffer.append(queryArgs[i]);
        }
        return queryStringBuffer.toString();
    }
}
