// Funes. Copyright (c) 2001, Nelson Minar <nelson@monkey.org>
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package org.monkey.nelson.funes.stubmail;

import javax.mail.internet.MimeMessage;
import javax.mail.*;
import java.io.InputStream;
import java.util.Properties;

/** A subclass of JavaMail's Message class that has a publically
 ** accessible constructor. This can be used to bypass all the
 ** JavaMail crap about folders, stores, and sessions, and just use it
 ** to parse email. StubStore and StubFolder are used to fake out the
 ** MimeMessage() constructors. Grrr!
 ** 
 ** @author Nelson Minar &lt;nelson@monkey.org&gt;
 **/
public class StubMessage extends MimeMessage {
    /** Construct a new Javamail message from the given input stream. **/
    public StubMessage(InputStream is) throws MessagingException {
        this(stubFolder, is, -1);
    }
    
    /** Create a new Message by parsing the input stream.
     ** Some parameters (folder, messageNumber) are not used by Funes,
     ** but have to be here to make the JavaMail APIs happy. Folder
     ** can safely be an instance of StubFolder, and messageNumber can
     ** safely always be "-1".
     **
     ** @param folder Ignored by Funes: StubFolder is safe.
     ** @param is InputStream for the message to read.
     ** @param messageNumber Ignored by Funes: -1 is safe.
     **/
    protected StubMessage(Folder folder, InputStream is, int messageNumber) throws MessagingException {
        super(folder, is, messageNumber);
    }

    // The stuff below is stub code to make Javamail happy.
    private static Session stubSession;
    private static Store stubStore;
    private static Folder stubFolder;

    // Initialize the stub Javamail objects. Requires a javamail.providers
    // file in the META-INF section of the jarfile!
    static {
        stubSession = Session.getDefaultInstance(new Properties());
        try {
            stubStore = stubSession.getStore("stub");
            stubFolder = stubStore.getDefaultFolder();
        } catch (Exception e) {
            throw new RuntimeException("javamail stub provider is missing - is javamail.providers in the jar?");
        }
    }
}
