// Funes. Copyright (c) 2001, Nelson Minar <nelson@monkey.org>
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package org.monkey.nelson.funes.test;
import junit.framework.*;
import java.io.*;
import javax.mail.*;
import org.apache.lucene.document.*;
import org.monkey.nelson.funes.stubmail.StubMessage;
import org.monkey.nelson.funes.*;

public class ParsedMessageTest extends TestCase {
    public ParsedMessageTest(String name) { super(name); }
    public static Test suite() { return new TestSuite(ParsedMessageTest.class); }
    public void testSimpleMessage() throws Exception {
        StubMessage m = new StubMessage(new ByteArrayInputStream(Messages.simpleMessage.getBytes()));
        ParsedMessage pm = new ParsedMessage(m);

        assertEquals(ParsedMessage.STATUS_OK, pm.getStatus());
        assertNull(pm.getException());
        
        Document d = pm.getDocument();
        Field f;

        assertEquals("Short message, minimal headers\n\n",
                     d.get(Schema.bodyName));
        assertEquals("short test message",
                     d.get(Schema.subjectName));
        assertEquals("<15232.9647.966199.669746@cabernet.nelson.monkey.org>",
                     d.get(Schema.messageIDName));
        assertEquals("alice@cypherpunks.org (Nelson Minar)",
                     d.get(Schema.authorName));
        assertEquals("nelson@monkey.org (Nelson Minar), bob@cypherpunks.org (Bob Dobbs), mallory@cypherpunks.org",
                     d.get(Schema.recipientsName));
        assertEquals("Sun Aug 19 13:46:39 PDT 2001",
                     DateField.stringToDate(d.get(Schema.dateName)).toString());
        // can't test Schema.locationName with StubMail.
    }
    
}
