// Funes. Copyright (c) 2001, Nelson Minar <nelson@monkey.org>
// All rights reserved. Distributed with no warranty, without even the
// implied warranty of merchantability or fitness for a particular purpose.
// For more details see COPYING.GPL, the GNU General Public License.

package org.monkey.nelson.protocol.xfilerange.test;
import junit.framework.*;
import org.monkey.nelson.protocol.xfilerange.*;
import java.io.*;
import java.net.*;

/**
 ** @author Nelson Minar &lt;nelson@monkey.org&gt;
 **/
public class XFileRangeURLConnectionTest extends TestCase {
    public XFileRangeURLConnectionTest(String name) { super(name); }
    public static Test suite() { return new TestSuite(XFileRangeURLConnectionTest.class); }

    // This code assumes that CWD is the root of the package.
    private final String testFileName = "src/org/monkey/nelson/protocol/xfilerange/test/testfile.txt";
    private String canonicalTestFileName;

    private String testContentString = "abcde\nghi\nkl\nn\n";
    
    public void setUp() throws Exception {
        Handler.install();
        canonicalTestFileName = new File(testFileName).getCanonicalPath();
    }

    public void testOpenAbsolute() throws Exception {
        String urlName = "xfilerange://localhost" + canonicalTestFileName;
        URL url = new URL(urlName);
        URLConnection uc = url.openConnection();
        assertEquals(XFileRangeURLConnection.class, uc.getClass());
        InputStream is = uc.getInputStream();
        assertTrue(is instanceof InputStream);
    }
    
    public void testFullRead() throws Exception {
        String urlName;
        InputStream is;

        // lots of different ways to name "the whole file".
        
        urlName = "xfilerange://localhost" + canonicalTestFileName;
        is = (new URL(urlName)).openStream();
        assertEquals(testContentString, readFully(is));

        urlName = "xfilerange://localhost" + canonicalTestFileName + "#-";
        is = (new URL(urlName)).openStream();
        assertEquals(testContentString, readFully(is));

        urlName = "xfilerange://localhost" + canonicalTestFileName + "#0-";
        is = (new URL(urlName)).openStream();
        assertEquals(testContentString, readFully(is));

        urlName = "xfilerange://localhost" + canonicalTestFileName + "#0-15";
        is = (new URL(urlName)).openStream();
        assertEquals(testContentString, readFully(is));

        urlName = "xfilerange://localhost" + canonicalTestFileName + "#-15";
        is = (new URL(urlName)).openStream();
        assertEquals(testContentString, readFully(is));
    }

    public void testMiddleToEnd() throws Exception {
        String urlName = "xfilerange://localhost" + canonicalTestFileName +
            "#7-";
        InputStream is = (new URL(urlName)).openStream();
        assertEquals(testContentString.substring(7), readFully(is));
    }

    public void testStartToMid() throws Exception {
        String urlName = "xfilerange://localhost" + canonicalTestFileName +
            "#-7";
        InputStream is = (new URL(urlName)).openStream();
        assertEquals(testContentString.substring(0, 7), readFully(is));
    }

    public void testMidToMid() throws Exception {
        String urlName = "xfilerange://localhost" + canonicalTestFileName +
            "#4-7";
        InputStream is = (new URL(urlName)).openStream();
        assertEquals(testContentString.substring(4, 7), readFully(is));
    }

    private String readFully(InputStream is) throws IOException {
        return ByteRangeInputStreamTest.readFully(is);
    }
}
